#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhumenu.h"

static void print_name (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "-");
		return;
	}
	while (*s != '\0') {
		if (*s == '\\' || *s == '"') fputc('\\', f);
		if (*s == '%' || *s == '&' || *s == '*') fputc(*s, f);
		fputc(*s++, f);
	}
}

static void print_exec (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "false");
		return;
	}
	while (*s != '\0') {
		if (*s != '%') { fputc(*s++, f); continue; }
		s++;
		if (*s == '%' || *s == '\0') fputc('%', f);
		if (*s != '\0') s++;
	}
}

static void dump (FILE *f, const struct item *item, const char *lang)
{
	int i;
	int major = 0;
	const struct menu *menu = item->submenu;
	fprintf(f, "DestroyMenu UHU%s\n", item->file_name);
	fprintf(f, "AddToMenu UHU%s \"", item->file_name);
	print_name(f, get_value_8bit(item, NULL, "Name", lang));
	fprintf(f, "\" Title\n");
	for (i = 0; i < menu->nr; i++) {
		if (i > 0 && menu->items[i].major != major) {
			fprintf(f, "+\t\"\" Nop\n");
		}
		fprintf(f, "+\t\"");
		print_name(f, get_value_8bit(&menu->items[i], NULL, "Name", lang));
		fprintf(f, "\" ");
		if (!menu->items[i].issubmenu) {
			fprintf(f, "Exec %s",
			  menu->items[i].terminal ? "xterm -e " : "");
			print_exec(f, get_value(&menu->items[i], NULL, "Exec", lang));
		} else {
			fprintf(f, "Popup UHU%s", menu->items[i].file_name);
		}
		fprintf(f, "\n");
		major = menu->items[i].major;
	}
	fprintf(f, "\n");
	for (i = 0; i < menu->nr; i++) {
		if (menu->items[i].issubmenu) {
			dump(f, &menu->items[i], lang);
		}
	}
}

int write_menu_lang (const struct menu *menu, const char *path,
  const struct language *language)
{
	char *lang;
	char file[PATH_MAX];
	FILE *f;

	lang = language->name;
	sprintf(file, "%s/menu%s%s", path, lang[0] ? "." : "", lang);
	if ((f = fopen(file, "w")) == NULL) {
		fprintf(stderr, "icewm: fopen %s: %s\n", file, strerror(errno));
		return -1;
	}
	if (menu->nr > 0) dump(f, &menu->items[0], lang);
	fclose(f);
	return 0;
}

